from F import LIST
import Sentences
import Words


def tokenize_content_into_sentences(content):
    """ Split a large string into sentences """
    sentences = Sentences.to_sentences(content)
    sentences = [x.replace('\n', '') for x in sentences if len(x) > 10]
    return sentences

def score_complete_tokenization(tokenization: dict):
    result = {}
    for key in tokenization.keys():
        token_list = tokenization[key]
        result[key] = Words.score_words(token_list)
    return result

# @Ext.safe_args
def complete_tokenization_v2(*content, toList=True):
    """ PUBLIC """
    content = LIST.flatten(content)
    toStr = LIST.to_str(content)
    tokens = Words.to_words_v1(toStr)
    bi_grams = Words.to_x_grams(tokens, 2)
    tri_grams = Words.to_x_grams(tokens, 3)
    quad_grams = Words.to_x_grams(tokens, 4)
    if toList:
        return tokens + bi_grams + tri_grams + quad_grams
    else:
        return {"tokens": tokens, "bi_grams": bi_grams, "tri_grams": tri_grams, "quad_grams": quad_grams}



import re

from F import LIST
from FNLP.Regex import Constants

FINDALL = lambda regex, content: re.findall(regex, content)
SEARCH = lambda regex, content: re.search(regex, content)

def search(search_term, content):
    match = FINDALL(Constants.SEARCH_TERM(search_term), content)
    return match if len(match) >= 1 and match is not None else False

def contains(search_term, content):
    try:
        if type(content) in [list, tuple]:
            for itemContent in content:
                match = re.findall(fr'.*{str(search_term)}.*', itemContent)
                if len(match) >= 1 and match is not None:
                    return True
            return False
        else:
            match = re.findall(fr'.*{str(search_term)}.*', content)
            return True if len(match) >= 1 and match is not None else False
    except Exception as e:
        print(f"Failed to regex findall. {str(search_term)}, error=[ {e} ]")
        return False

def contains_strict(search_term, content):
    try:
        if type(content) in [list, tuple]:
            for itemContent in content:
                search_term = __remove_special_characters(search_term)
                match = re.findall(fr'\b{search_term}\b', itemContent)
                if len(match) >= 1 and match is not None:
                    return True
            return False
        else:
            search_term = __remove_special_characters(search_term)
            match = re.findall(fr'\b{search_term}\b', content)
            return True if len(match) >= 1 and match is not None else False
    except Exception as e:
        print(f"Failed to regex findall. {search_term}, error=[ {e} ]")
        return False

def contains_any(search_terms, content):
    search_terms = LIST.flatten(search_terms)
    for term in search_terms:
        temp = contains(term, content)
        if temp:
            return True
    return False

def locate_term_in_str(term, content):
    match = re.search(fr'\b{term}\b', content)
    if match:
        return match
    return False


def __remove_special_characters(text):
    """ DEPRECATED """
    newText = re.sub('[^a-zA-Z0-9]', ' ', text)
    return newText




def to_paragraphs(body: str) -> [str]:
    """ -> Separates text based on "\n" <- """
    paragraph_list = []
    i = 0
    temp_body = body
    for char in body:
        if char == "\n":
            new_body = temp_body[:i]
            paragraph_list.append(new_body)
            temp_body = temp_body[i+1:]
            i = 0
            continue
        i += 1
    return paragraph_list


import re
from F import LIST
from F import DATE
from . import Re

def str_contains_year(content):
    match = re.findall(r'\b[1-2][0-9][0-9][0-9]\b', content)
    if match and len(match) >= 1:
        if LIST.get(0, match) == "":
            return False

        newMatch = []
        for year in match:
            if len(year) > 4:
                continue
            if int(year) > int(DATE.get_current_year()):
                continue
            if int(year) < 1900:
                continue

            newMatch.append(year)
        return newMatch

def extract_date(content):
    """ EXPERIMENTAL AND NOT FINISHED! """
    if type(content) not in [str]:
        content = str(content)
    all = []
    months = DATE.months.keys()
    for item in months:
        all.append(item)
    month_variants = DATE.months.values()
    for item in month_variants:
        all.append(item)
    temp_all = LIST.flatten(all)
    _all = LIST.remove_duplicates(temp_all)
    for month_variant in _all:
        # -> Check for any Month or Month Variant in Full String
        locate_month_variant = Re.locate_term_in_str(month_variant, content)
        if locate_month_variant:
            start = locate_month_variant.start()
            end = start + 15
            temp = content[start:end]
            # -> Check for Year immediately after Month
            find_year = Re.str_contains_year(temp)
            year = LIST.get(0, find_year)
            locate_year = Re.locate_term_in_str(year, temp)
            if locate_year:
                year_end = locate_year.end()
                temp2 = content[start:start+year_end]
                # -> Clean Result.
                filter = Re.__remove_special_characters(temp2)
                result = filter.replace("  ", " ").strip()
                return result
    return False


from . import Constants
from F import LIST

def is_in_alphabet_lower(content:str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar in Constants.ALPHABET_LOWER:
        return True
    return False

def is_in_alphabet_upper(content:str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar in Constants.ALPHABET_UPPER:
        return True
    return False

def is_in_alphabet(content:str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar in Constants.ALPHABET_ALL:
        return True
    return False

def is_single_number(content):
    if type(content) != int:
        content = LIST.get(0, content, default=False)
    if content in Constants.NUMBERS_SINGLE:
        return True
    return False

def is_capital(content: str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar and str(firstChar).isupper():
        return True
    return False

def are_capital(*content: str):
    for item in content:
        if not is_capital(item):
            return False
    return True

def are_periods(*content: str):
    for item in content:
        if not is_period(item):
            return False
    return True

def is_period(content:str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar and str(content) == ".":
        return True
    return False

def are_periods_or_capitals(*content:str):
    for item in content:
        if is_capital(item) or is_period(item):
            return True
    return False

def is_empty(content: str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar and not content or content == ' ' or content == '' or str(content) == " ":
        return True
    return False

def are_empty(*content: str):
    for item in content:
        if not item or item == ' ' or item == '' or str(item) == " ":
            return True
    return False

def is_quotation(content:str):
    encoded_character = str(content).encode('utf-8')
    if content == '"':
        return True
    elif encoded_character in Constants.QUOTES_ENCODINGS:
        return True
    return False

def is_space(content:str):
    firstChar = LIST.get(0, content, default=False)
    if firstChar and str(content) == ' ':
        return True
    return False

def is_space_or_quotation(content):
    if is_quotation(content) or is_space(content):
        return True
    return False

def __is_sentence_ender(content):
    if str(content) in Constants.SENTENCE_ENDERS:
        return True
    return False

def __is_sentence_beginner(content):
    if is_in_alphabet(content):
        return True
    elif is_quotation(content):
        return True
    elif is_single_number(content):
        return True
    return False


from F import DICT

from FA.Categories import Topics
import FairResources

WEIGHTED_TERMS = Topics.ALL_CATEGORIES().get_all_weighted_terms()

STOP_WORDS = FairResources.get_stopwords()

SENTENCE_ENDERS = ['.', '?', '!']
QUOTES_ENCODINGS = [b'\xe2\x80\x9e', b'\xe2\x80\x9f', b'\xe2\x80\x9d', b'\xe2\x80\x9c']

ALPHABET_DICT_PAIRS = {"a": "A", "b": "B", "c": "C", "d": "D", "e": "E", "f": "F", "g": "G", "h": "H",
                       "i": "I", "j": "J", "k": "K", "l": "L", "m": "M", "n": "N", "o": "O", "p": "P",
                       "q": "Q", "r": "R", "s": "S", "t": "T", "u": "U", "v": "V", "w": "W", "x": "X",
                       "y": "Y", "z": "Z" }

GET_CAPITAL_FROM_LOWER = lambda lowerChar: DICT.get(lowerChar, ALPHABET_DICT_PAIRS, default=False)
GET_LOWER_FROM_CAPITAL = lambda capitalChar: DICT.get_key(capitalChar, ALPHABET_DICT_PAIRS, default=False)
GET_OPPOSITE_LOWER_OR_UPPER = lambda char: GET_CAPITAL_FROM_LOWER(char) if str(char).islower() else GET_LOWER_FROM_CAPITAL(char)

ALPHABET_LOWER = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m",
                  "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"]
ALPHABET_UPPER = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
                  "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"]
ALPHABET_ALL = ALPHABET_LOWER + ALPHABET_UPPER
NUMBERS_SINGLE = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]

SUMMARY = lambda first, middle, last: f"{first} {middle} {last}"


from F import LIST
from FNLP.Language import Constants
from FNLP.Language import Utils, Character
from FNLP import Regex

def find_proper_nouns(content:str):
    return Regex.extract_only_capital_words_regex(content)

def make_capital(word:str):
    firstChar = word[0]
    newWord = word[1:]
    if Character.is_in_alphabet(firstChar) and not Character.is_capital(firstChar):
        firstChar = Constants.GET_CAPITAL_FROM_LOWER(firstChar)
    newWord = firstChar + newWord
    return newWord

def make_lower(word:str):
    firstChar = word[0]
    newWord = word[1:]
    if Character.is_in_alphabet(firstChar) and Character.is_capital(firstChar):
        firstChar = Constants.GET_LOWER_FROM_CAPITAL(firstChar)
    newWord = firstChar + newWord
    return newWord

def to_x_grams(tokens, x):
    """ PUBLIC """
    if type(tokens) == str:
        tokens = to_words_v1(tokens)
    i = 0
    x_grams = []
    if len(tokens) < x:
        print("found none", tokens)
        return x_grams
    for _ in tokens:
        if i+x > len(tokens):
            break
        phrase = ""
        for c in range(x):
            phrase = combine_words(phrase, tokens[i+c])
        x_grams.append(phrase)
        i += 1
    return x_grams

def to_words_v1(content: str):
    content = Utils.replace(content, ".", ",", ";", "\n", "  ")
    s = content.split(" ")
    newS = Utils.remove_empty_strings(s)
    return newS

def to_bi_grams_v2(tokens):
    """ PUBLIC HELPER """
    return to_x_grams(tokens, 2)

def to_tri_grams_v2(tokens):
    """ PUBLIC HELPER """
    return to_x_grams(tokens, 3)

def to_quad_grams_v2(tokens):
    """ PUBLIC HELPER """
    return to_x_grams(tokens, 4)

def expand_word(word: str) -> []:
    """ PUBLIC -> FAIR Expansion <- """
    word_lower = word.lower()
    word_upper = word.upper()
    word_first_capital = word[0].upper() + word[1:]
    # word_stem = lemmatize_word(word)
    # word_stem_first_capital = word_stem[0].upper() + word_stem[1:]
    return [word, word_lower, word_upper, word_first_capital]

def score_words(words):
    result = {}
    for word in words:
        if word in result.keys():
            tempValue = result[word]
            result[word] = tempValue + 1
        else:
            result[word] = 1
    return result

def combine_words(*words):
    """ Combines two strings together. """
    temp_word = ""
    words = LIST.flatten(words)
    if len(words) > 0:
        for word in words:
            temp_word += " " + word.strip()
        return temp_word.strip()
    return str(words).strip()

def remove_ing(word):
    if word.endswith("ing"):
        return word[:-3]
    elif word.endswith("ings"):
        return word[:-4]
    return False

def remove_apos(word):
    word = word.replace("'", "")
    return word

def __split_words(text):
    """ ALTERNATIVE: Split a string into array of words. """
    try:
        import re
        text = re.sub(r'[^\w ]', '', text)  # strip special chars
        return [x.strip('.').lower() for x in text.split()]
    except TypeError:
        return None


import re
import requests
from FNLP.Regex import Re as Regex
from F import LIST

"""
    -> URL HANDLING
-> One period vs Two Periods?
    - The idea is all URLs have either 1 or 2 periods in the string.
        1. "verge.com"
        2. "www.verge.com"
        - Outside of this, there are only slashes. No more periods.
    - We figure out which it is in the current url
        - if 1 period, sitename is before last period.
        - if 2 periods, sitename is inbetween two periods.
    - We then parse the url to find the base site name.
"""

def get_base_url(url):
    """ -> PUBLIC -> Extract Base Site Name from URL <- """
    try:
        url = remove_http(url)
        last_index = 0
        for char in url:
            if str(char) == "/":
                base = url[:last_index]
                return base
            last_index += 1
        return url
    except Exception as e:
        print(f"Failed to get base url. URL=[ {url} ], error=[ {e} ]")
        return url

# @DeprecationWarning
def get_site_name(url):
    """ -> DEPRECATED FOR BASE_URL -> Extract Base Site Name from URL <- """
    if verifyTwoPeriods(url):
        return extract_siteName_two_periods(url)
    return extract_siteName_one_period(url)

def remove_http(url):
    return url.replace("https://", "").replace("http://", "")

def extract_base_url(url: str):
    if Regex.contains_any(search_terms=["http://", "https://"], content=url):
        newUrl = url.replace("https://", "").replace("http://", "")
        index = 0
        last = len(newUrl)
        for char in newUrl:
            if index >= last-1:
                return newUrl.replace("/", "")
            if char == "/":
                return newUrl[:index].replace("/", "")
            index += 1
    else:
        index = 0
        last = len(url)
        for char in url:
            if index >= last-1:
                return url.replace("/", "")
            if char == "/":
                return url[:index].replace("/", "")
            index += 1
    return url

def extract_sub_reddit(url):
    slash_count = 0
    index = 0
    start = 0
    end = 0
    url = remove_http(url)
    for char in url:
        if char == "/":
            slash_count += 1
            if slash_count == 2:
                # do work
                start = index - 1
            if slash_count == 3 or index >= len(url):
                end = index
        index += 1
    return url[start:end]

# -> get_site_name HELPER for verifying URL has one or two periods.
def verifyTwoPeriods(url: str):
    """ PRIVATE """
    count = 0
    for char in url:
        if char == '.':
            count += 1
    if count >= 2:
        return True
    return False


# -> get_site_name HELPER for urls with one period.
def extract_siteName_one_period(url):
    """ PRIVATE """
    i = 0
    slash_count = 0
    removal_index = 0
    for char in url:
        if char == "/":
            slash_count += 1
            if slash_count == 2:
                removal_index = i + 1
        if char == '.':
            return url[removal_index:i]
        i += 1
    return url


# -> get_site_name HELPER for urls with two periods.
def extract_siteName_two_periods(url):
    """ PRIVATE """
    temp = ""
    start = 0
    end = 0
    i = 0
    periodCount = 0
    lastChar = ''
    # -> Part 1
    for char in url:
        if char == '/' and lastChar == '/':
            start = i + 1
        if char == '.':
            periodCount += 1
            if periodCount == 2:
                end = i
        if end > 0:
            temp = url[start:end]
            break
        lastChar = char
        i += 1
    # -> Part 2
    n = 0
    match = ""
    for c in temp:
        if c == '.':
            match = temp[n + 1:]
        n += 1
    return match


def extract_data_src_url(content: str):
    """ EXPERIMENTAL (under development still) """
    new = content.split("https://")
    for item in new:
        test = Regex.contains(":", item)
        if test:
            continue
        if len(item) < 5:
            continue
        return "https://" + item
    return False


def is_valid_url(url):
    """ PUBLIC """
    try:
        response = requests.get(url)
        if response:
            print("URL is valid and exists on the internet")
            return True
    except requests.ConnectionError as e:
        print("URL does not exist on Internet, error=[ {e} ]")
        return False

def is_url(content: str):
    """ PUBLIC """
    try:
        if type(content) in [list, tuple]:
            for itemContent in content:
                match = re.search(r'http.?://.*/', add_http(itemContent))
                if match is not None:
                    return True, match.string
            return False, False
        else:
            match = re.search(r'http.?://.*/', add_http(content))
            return True, match.string if match is not None else False
    except Exception as e:
        print(f"Failed to regex findall. {content}, error=[ {e} ]")
        return False, False

def url_ends_with(url, *args):
    args = LIST.flatten(args)
    for item in args:
        if str(url).endswith(item):
            return True
    return False

bad_exts = [".png", ".jpg", ".jpeg", ".gif", ".pdf", ".css", ".js", ".webp", ".ico", ".mjs", "svg", ".woff2"]
def filter_out_bad_exts(urls):
    filtered_urls = []
    for inner_url in urls:
        if url_ends_with(inner_url, bad_exts):
            continue
        filtered_urls.append(inner_url)
    return filtered_urls

def find_urls_in_str(content: str):
    if type(content) is not str:
        content = str(content)
    match = re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', content)
    newMatch = []
    if match:
        for item in match:
            newItem = clean_url(item)
            newMatch.append(newItem)
    return newMatch

def clean_url(url: str):
    if type(url) is not str:
        url = str(url)
    newUrl = url.replace("\'", "").replace(",", "").replace("\"", "")
    return newUrl


def add_http(content):
    if content.startswith("//") or not content.startswith("http"):
        url1 = "https:" + content
        return url1
    return content


def avoid_url(url, avoid_list):
    """ EXPERIMENTAL """
    base_url = extract_base_url(url)
    if Regex.contains_any(avoid_list, base_url):
        return True
    return False


def filter_out_avoid_list(urls: [], avoid_list: []):
    """ EXPERIMENTAL """
    filtered_urls = []
    for single_url in urls:
        base_url = extract_base_url(str(single_url))
        if Regex.contains_any(avoid_list, base_url):
            continue
        filtered_urls.append(single_url)
    return filtered_urls


if __name__ == '__main__':
    test = get_site_name("https://www.cnet.co/news/")
    print(test)


from FA.Categories import Topics
from F import LIST
from . import Sentences
from . import Words
from FNLP.Language import Constants

# -> master summarizer!
def summarize(content='', max_sents=5):
    if not content or max_sents <= 0:
        return []
    keepList = []
    # Pre. -> Convert raw string of text into a List of Sentences.
    sentences = Sentences.to_sentences(content)
    if not sentences:
        return False
    # 1. -> If only 6 or less sentences to start, return now
    if len(sentences) <= 6:
        return Words.combine_words(sentences)
    # 2. -> Always use the first and last sentence
    firstSentence = LIST.get(0, sentences, False)
    if not firstSentence:
        return False
    lastIndex = len(keepList) - 1
    lastSentence = LIST.get(lastIndex, sentences, False)
    if len(lastSentence) <= 50:
        lastSentence = LIST.get(lastIndex - 1, sentences, False)
    # 3. -> Remove First and Last Sentence
    without_first = LIST.remove_index(0, sentences)
    without_first_and_last = without_first[:-1]
    # 4. -> Filter out by length
    for sen in without_first_and_last:
        l = len(sen)
        if 50 < l > 400:
            continue
        keepList.append(sen)
    # 5. -> Section off list into three parts.
    #           - Beginning, Middle, End.
    base_count = int(len(keepList) / 3)
    middle_count = base_count * 2
    first = keepList[:base_count]
    middle = keepList[base_count:middle_count]
    last = keepList[middle_count:]
    # 6. -> Score each Section
    first_scored = Topics.ALL_CATEGORIES().score_categorizer(first)
    middle_scored = Topics.ALL_CATEGORIES().score_categorizer(middle)
    last_scored = Topics.ALL_CATEGORIES().score_categorizer(last)
    # 7. -> Filter/Select highest scored sentences from each Section.
    first_summary = form_summary(first_scored, 1)
    middle_summary = form_summary(middle_scored, 1)
    last_summary = form_summary(last_scored, 1)
    # 8. -> Combine all 3 Sections into 1 Single Body of Text.
    combined_summary = Words.combine_words(first_summary, middle_summary, last_summary)
    # 9. -> Combine the first sentence, the middle body and the last sentence to form "The_Summary"
    The_Summary = Constants.SUMMARY(firstSentence, combined_summary, lastSentence)
    return The_Summary

def form_summary(scored_sentences: [], max_sent=5):
    final_list = []
    total_count = len(scored_sentences) - 1
    current_index = 0
    sorted_scored_sentences = sorted(scored_sentences, key=lambda lst: lst[0], reverse=True)
    while current_index <= total_count:
        if len(final_list) >= max_sent:
            break
        raw_sent = LIST.get(current_index, sorted_scored_sentences)
        sent = LIST.get(1, raw_sent)
        # - Finish up
        final_list.append(sent)
        current_index += 1
    the_summary = Words.combine_words(final_list)
    return the_summary